--*******************************************************************************************************************************
--
-- Description: This module is used to synchronize the frame data produced by multiple ADC chips that have a high speed
--				serial data interface.  The divider state machine (DSM) clears the divider in the BUFR connected to data
--				clock so that the frames across multiple devices can achieve phase coherence.  The align state machine (ASM)
--				uses the ISERDES bit slip to align the frame for an individual device and then enables the data flow for
--				all devices simultaneously to obtain phase coherence across all devices.
--					
-- Rev History: 04-17-2017:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM;
use UNISIM.Vcomponents.all;

entity frame_lock is
port (
	-- Reset and clocks supplied by the application logic.
    data_srst		: in std_logic;		-- Active high synchronous reset in the data clock domain.
    data_div_srst	: in std_logic;		-- Active high synchronous reset in the divided data clock domain.
	data_clk		: in std_logic;		-- Serial data clock into the ISERDES.
	data_clk_div	: in std_logic;		-- Divided serial data clock (above).
	frame_re		: in std_logic;		-- Rising edge sample of frame input.

	-- Signals used to trigger and monitor the frame synchronization function.
	lock_start		: in std_logic; 	-- Logic one initiates the frame synchronization sequence.
	lock_status		: out std_logic;	-- Logic one indicates that a solution was found.
		
	-- Connections to the BUFR, ISERDES, and FIFO elements.
	clk_div_clr		: out std_logic; 	-- Active high clears to the BUFR divider.
	clk_div_enable	: out std_logic; 	-- Active high enables the BUFR divider.
	iserdes_rst		: out std_logic; 	-- Active high resets the frame ISERDES.
	frame_detect	: in  std_logic_vector(7 downto 0); -- Current output byte from the frame ISERDES.
	frame_bitslip	: out std_logic; 	-- Active high bit slip enable to the ISERDES.
	multisync_req	: out std_logic; 	-- Logic one indicates that the ISERDES frame is aligned and waiting for multisync.
	multisync		: in std_logic; 	-- Logic one indicates that all devices will synchronize on the next clock.
	fifo_rst		: out std_logic; 	-- Active high FIFO asynchronous reset.
	fifo_wren		: out std_logic 	-- Active high FIFO write enable.
);
end frame_lock;

architecture behavioral of frame_lock is

	type DSM is (dsm_idle, dsm_enable, dsm_clear1, dsm_stable1, dsm_ready, dsm_clear2, dsm_phase, dsm_stable2, dsm_fail);	-- Type definitions for a state machine to clear the BUFR divider.
	signal dsm_state 		: DSM;																	-- Current state.
	signal last_dsm_state	: DSM;																	-- State on previous clock.
	
	type ASM is (asm_idle, asm_reset, asm_settle, asm_compare, asm_bitslip, asm_ready, asm_fail);	-- Type definitions for a state machine to correctly align the ISERDES output word.
	signal asm_state 		: ASM;																	-- Current state.
	signal last_asm_state	: ASM;																	-- State on previous clock.
	
	signal dsm_timer			: unsigned(3 downto 0);		-- General purpose timer on the DSM clock domain.
	signal asm_timer			: unsigned(2 downto 0);		-- General purpose timer on the ASM clock domain.
	signal bit_counter			: unsigned(2 downto 0);		-- Counter to track the eight possible bit orderings in a frame.
	signal divide_busy			: std_logic;				-- Logic one indicates that the clear state machine is active.
	signal divide_busy_strobe	: std_logic;				-- Single period strobe of the busy signal (above) in the divided data clock domain.
	signal divide_fail			: std_logic;				-- Logic one indicates that the clear process failed to detect a frame clock.
	signal divide_fail_strobe	: std_logic;				-- Single period strobe of the fail signal (above) in the divided data clock domain.
	signal align_start			: std_logic;				-- Logic one initiates the frame alignment process from the clear state machine.
	signal align_start_strobe	: std_logic;				-- Single period strobe of the start signal (above) in the divided data clock domain.
	signal align_busy			: std_logic;				-- Logic one indicates that the align state machine is active.
	signal multisync_rclk		: std_logic;				-- Multisync control bit reclocked in the divided data clock domain.
	
begin                    

--****************************************** DIVIDER STATE MACHINE (DSM) TRANSITIONS *******************************************

	process(data_clk)
	begin
		if rising_edge(data_clk) then

			if data_srst = '1' then
				dsm_state <= dsm_idle;
				
			else
				
				-- The following state machine resets the BUFR divider for frame synchronization across multiple ADC chips.
				case dsm_state is	
				
					-- Remain idle while waiting for the application logic to initiate a frame sync.
					when dsm_idle =>
						-- Detect the end of the clock lock process before resetting the BUFR divider.	
						if lock_start = '1' then
							dsm_state <= dsm_enable;
						end if;
						
					-- Enable the BUFR clock output.
					when dsm_enable =>
						-- Wait for the active clock output to stabilize.
						if dsm_timer = x"F" then
							dsm_state <= dsm_clear1;
						end if;
						
					-- Assert the clear input to the data clock dividing BUFR to reset the divider.
					when dsm_clear1 =>
						-- Hold the clear active.
						if dsm_timer = x"F" then
							dsm_state <= dsm_stable1;
						end if;
						
					-- Wait for the divided clock to stabilize.
					when dsm_stable1 =>
						-- Allow the clock to flush the downstream ISERDES.
						if dsm_timer = x"F" then
							dsm_state <= dsm_ready;
						end if;
						
					-- Wait here after the BUFR divider is first phase synchronized to to the frame clock (below).
					when dsm_ready =>
						-- Continue if this is the first pass through the ready state otherwise wait for the end of the clock lock process.	
						if last_dsm_state = dsm_stable1 or lock_start = '1' then
							dsm_state <= dsm_clear2;
						end if;
						
					-- Assert the clear input to the data clock dividing BUFR to reset the divider.
					when dsm_clear2 =>
						-- Wait for the last phase of frame (low) after a minumum two clock periods of active clear.
						if dsm_timer > x"1" and frame_re = '0' then
							dsm_state <= dsm_phase;
						end if;
						
					-- Align the internal frame clock edge with the leading edge of external frame clock or one serial clock period early.
					-- Note: The frame signal has already been delayed one serial clock period by reclocking.
					when dsm_phase =>
						-- The entire clear cycle, including stabilze, should take less than sixteen clock periods.
						if dsm_timer = x"F" then
							dsm_state <= dsm_fail;
						-- Wait for the second phase of frame (low).
						elsif frame_re = '0' then
							dsm_state <= dsm_stable2;
						end if;
						
					-- Wait for the divided clock to stabilize.
					when dsm_stable2 =>
						-- The entire clear cycle, including stabilze, should take less than sixteen clock periods.
						if dsm_timer = x"F" then
							dsm_state <= dsm_ready;
						end if;
						
					-- Set the error bit if both phases of the frame could not be detected in sixteen clocks.
					when dsm_fail =>
						dsm_state <= dsm_idle;
						
					-- Define the default state to be null.
					when others =>
						null;
					
				end case;
			end if;
		end if;
	end process;
	
--******************************************** DIVIDER STATE MACHINE (DSM) EVENTS **********************************************
	
	-- Set the fail bit when the state machine lands in the fail state due to a timeout.
	divide_fail <= '1' when dsm_state = dsm_fail else '0';

	-- Set the busy bit when the state machine is active.
	divide_busy <= '0' when dsm_state = dsm_idle or (dsm_state = dsm_ready and last_dsm_state /= dsm_stable1) else '1';
	
	-- Set the BUFR enable control to activate the clock.
	clk_div_enable <= '0' when dsm_state = dsm_idle else '1';

	-- Set the BUFR clear control to reset the divider.
	clk_div_clr <= '1' when dsm_state = dsm_clear1 or dsm_state = dsm_clear2 or dsm_state = dsm_phase else '0';

	-- Start the frame alignment process only if the clear process was successful.
	align_start <= '1' when dsm_state = dsm_ready and last_dsm_state /= dsm_stable1 else '0';

	process(data_clk)
	begin
		if rising_edge(data_clk) then
		
			-- Set the last state to the current state.
			last_dsm_state <= dsm_state;
			
			-- Create a counter that can be used as a timer.
			if dsm_state = dsm_idle or dsm_state = dsm_ready then
				dsm_timer <= (others => '0');
			else
				dsm_timer <= dsm_timer + x"1";
			end if;

		end if;
	end process;
	
--******************************************* ALIGN STATE MACHINE (ASM) TRANSITIONS ********************************************

	-- Generate a single period fail strobe in the divided data clock domain.
	FAIL_XING : entity work.sync_enable
	port map (
		source_srst			=> data_srst,
		source_clk			=> data_clk,
		source_enable		=> divide_fail,
		target_srst			=> data_div_srst,
		target_clk  		=> data_clk_div,
		target_enable		=> divide_fail_strobe
	);

	-- Generate a single period start strobe in the divided data clock domain.
	START_XING : entity work.sync_enable
	port map (
		source_srst			=> data_srst,
		source_clk			=> data_clk,
		source_enable		=> align_start,
		target_srst			=> data_div_srst,
		target_clk  		=> data_clk_div,
		target_enable		=> align_start_strobe
	);

	process(data_clk_div)
	begin
		if rising_edge(data_clk_div) then

			if data_div_srst = '1' then
				asm_state <= asm_idle;
				
			else
				
				-- The following state machine searches for aligned bits (xF0) from the frame ISERDES output.
				case asm_state is	
				
					-- Remain idle while waiting for the clear state machine cycle to complete.
					when asm_idle =>
						-- Detect the end of the BUFR clear process before starting frame alignment.	
						if align_start_strobe = '1' then
							asm_state <= asm_reset;
						elsif divide_fail_strobe = '1' then
							asm_state <= asm_fail;
						end if;
						
					-- Reset the ISERDES and data FIFO.
					when asm_reset =>
						if asm_timer = "111" then
							asm_state <= asm_settle;
						end if;
						
					-- Wait for the ISERDES output to settle.
					when asm_settle =>
						if asm_timer = "111" then
							asm_state <= asm_compare;
						end if;
						
					-- Compare the frame ISERDES output to the desired value (xF0).
					when asm_compare =>
						if frame_detect = x"F0" then
							asm_state <= asm_ready;
						elsif bit_counter = "111" then
							asm_state <= asm_fail;
						else
							asm_state <= asm_bitslip;
						end if;
						
					-- Bit slip the frame ISERDES output if it is not yet aligned.
					when asm_bitslip =>
						asm_state <= asm_settle;
						
					-- Wait for all ADC chips to align before sending valid data to the FIFO.
					when asm_ready =>
						if multisync_rclk = '1' then
							asm_state <= asm_idle;
						end if;
						
					-- Set the error bit if none of the eight possible frame results match the desired value (xFO).
					when asm_fail =>
						asm_state <= asm_ready;
						
					-- Define the default state to be null.
					when others =>
						null;
						
				end case;
			end if;
		end if;
	end process;

--********************************************* ALIGN STATE MACHINE (ASM) EVENTS ***********************************************

	-- Reset the ISERDES to initialize the bitslip position.
	iserdes_rst <= '1' when asm_state = asm_reset else '0';
	
	-- Enable the ISERDES bit slip function if the current output does not match the desired value (xF0).
	frame_bitslip <= '1' when asm_state = asm_bitslip else '0';
	
	-- Assert FIFO reset before frame alignment begins.
	fifo_rst <= '1' when asm_state = asm_reset else '0';
	
	-- Assert FIFO write enable when the align process is idle.
	fifo_wren <= '1' when asm_state = asm_idle else '0';

	process(data_clk_div)
	begin
		if rising_edge(data_clk_div) then
		
			-- Set the last state to the current state.
			last_asm_state <= asm_state;
				
			-- Create a counter that can be used as a timer.
			if asm_state = asm_idle then
				asm_timer <= (others => '0');
			else
				asm_timer <= asm_timer + "001";
			end if;
	
			-- Count the number of different bit orderings that have been evaluated.
			if asm_state = asm_idle then
				bit_counter <= (others => '0');
			elsif asm_state = asm_bitslip then
				bit_counter <= bit_counter + "001";
			end if;
	
			-- Set a flag to let other devices know that this device is ready to complete synchronization.
			-- Note: Set the flag if the clear state machine failed so that other devices can complete.
			if asm_state = asm_idle then
				multisync_req <= '0';
			elsif asm_state = asm_ready then
				multisync_req <= '1';
			end if;
				
			-- Reclock the multisync control bit.
			multisync_rclk <= multisync;
	
			-- Set a fail status if either the divider or align state machines do not reach a solution.
			-- Note: A divider state machine failure triggers the align state machine failure.
			if asm_state = asm_idle then
				lock_status <= '1';
			elsif asm_state = asm_fail then
				lock_status <= '0';
			end if;
	
		end if;
	end process;

end behavioral;
